@@ -16,21 +16,37 @@ from . import transformationtypes
 _dummyData = _pd.DataFrame([None])
 
 
-def get_meta(df):
+def get_meta(df, large_threshold=1000):
     """
     extract the metadata from a dataframe needed to hand over to the Filter
     """
 
     def parse(key, val):
         if isinstance(val, _pd.CategoricalDtype):
+
+            if len(val.categories) > large_threshold:
+                return {
+                    "type": "categorical",
+                    "large": True,
+                    "cat": []
+                }
             return {
                 "type": "categorical",
                 "cat": val.categories.tolist()
             }
         elif val == dtype('O'):
+
+            cat = df[key].unique()
+            if len(cat) > large_threshold:
+                return {
+                    "type": "categorical",
+                    "large": True,
+                    "cat": []
+                }
+
             return {
                 "type": "categorical",
-                "cat": df[key].unique().tolist()
+                "cat": cat.tolist()
             }
         elif val == dtype('bool'):
             return {
@@ -47,7 +63,7 @@ def get_meta(df):
     }
 
 
-def get_meta_dask(df):
+def get_meta_dask(df, large_threshold=1000):
     from numpy import dtype
     import dask
     """
@@ -56,9 +72,18 @@ def get_meta_dask(df):
 
     def parse(key, val):
         if val == dtype('O'):
+
+            cat = df[key].unique()
+            if len(cat) > large_threshold:
+                return {
+                    "type": "categorical",
+                    "large": True,
+                    "cat": []
+                }
+
             return {
                 "type": "categorical",
-                "cat": df[key].unique()
+                "cat": cat
             }
         elif val == dtype('bool'):
             return {
@@ -101,8 +126,6 @@ def get_meta_dask(df):
 
 def get_plot(inputDataFrame, config, apply_parameterization=True):
 
-    print(config)
-
     errorResult = "Empty plot"
 
     try:
@@ -188,7 +211,9 @@ def get_plot(inputDataFrame, config, apply_parameterization=True):
                         (c if type(c) == list else [c]) for c in [
                             plotConfigData["params"].get(key) for key in
                             ["x", "y", "error_x", "error_y", "dimensions",
-                                "color", "facet_col", "facet_row", "hover_name"]
+                                "color", "facet_col",
+                                "size", "symbol", "pattern_shape",
+                                "facet_row", "hover_name"]
                         ] if c is not None
                         ] for b in a]
 
@@ -203,11 +228,9 @@ def get_plot(inputDataFrame, config, apply_parameterization=True):
                             el, inputDataFrame)
                     )
 
-
             usedCols = list(set(usedCols))
 
             # if we don't get a pandas dataframe it might be dask or mongodf
-            print(usedCols)
             if not isinstance(inputDataFrame, _pd.DataFrame):
                 try:
                     inputDataFrame = inputDataFrame[[c for c in usedCols if c in inputDataFrame.columns]].compute(
@@ -230,6 +253,19 @@ def get_plot(inputDataFrame, config, apply_parameterization=True):
                     inputDataFrame = getattr(
                         transformationtypes, el["type"]).compute(el, inputDataFrame)
 
+            # extract the render info from the params
+            if "render" in plotConfigData["params"]:
+                render_type = plotConfigData["params"]["render"]
+                del plotConfigData["params"]["render"]
+            else:
+                render_type = "auto"
+
+            if "render_size" in plotConfigData["params"]:
+                render_size = plotConfigData["params"]["render_size"]
+                del plotConfigData["params"]["render_size"]
+            else:
+                render_size = [1200, 700]
+
             # if we want to force an axis as categorical
             markCatX = False
             markCatY = False
@@ -243,12 +279,23 @@ def get_plot(inputDataFrame, config, apply_parameterization=True):
             # if we have cols and rows, we want to make independent
             makeIndepX = False
             makeIndepY = False
-            if "indep_x" in plotConfigData["params"]:
-                makeIndepX = True
-                del plotConfigData["params"]["indep_x"]
-            if "indep_y" in plotConfigData["params"]:
-                makeIndepY = True
-                del plotConfigData["params"]["indep_y"]
+            if plotConfigData["type"] not in ["imshow"]:
+                if "indep_x" in plotConfigData["params"]:
+                    makeIndepX = True
+                    del plotConfigData["params"]["indep_x"]
+                if "indep_y" in plotConfigData["params"]:
+                    makeIndepY = True
+                    del plotConfigData["params"]["indep_y"]
+
+            # if we have reversed x and y axis
+            reversedX = False
+            reversedY = False
+            if "reversed_x" in plotConfigData["params"]:
+                reversedX = True
+                del plotConfigData["params"]["reversed_x"]
+            if "reversed_y" in plotConfigData["params"]:
+                reversedY = True
+                del plotConfigData["params"]["reversed_y"]
 
             # add a sort command for categorical x columns
             if markCatX and "x" in plotConfigData["params"]:
@@ -287,9 +334,7 @@ def get_plot(inputDataFrame, config, apply_parameterization=True):
                 cat_values = inputDataFrame[val_name].unique()
                 cat_values.sort()
                 plotConfigData["params"]["category_orders"][val_name] = cat_values.tolist(
-                )                                
-
-
+                )
 
             # remove nan values from dataframe
             # inputDataFrame = inputDataFrame.dropna(subset=[c for c in inputDataFrame.columns if c in usedCols])
@@ -307,8 +352,6 @@ def get_plot(inputDataFrame, config, apply_parameterization=True):
             if plotConfigData["type"] == "density_contour":
                 fig = _px.density_contour(
                     inputDataFrame, **plotConfigData["params"])
-            if plotConfigData["type"] == "box":
-                fig = _px.box(inputDataFrame, **plotConfigData["params"])
             if plotConfigData["type"] == "violin":
                 fig = _px.violin(inputDataFrame, **
                                  plotConfigData["params"])
@@ -347,8 +390,18 @@ def get_plot(inputDataFrame, config, apply_parameterization=True):
             if makeIndepY:
                 fig.update_yaxes(matches=None, showticklabels=True)
 
+            # reverse the axis
+            if reversedX:
+                fig.update_xaxes(autorange="reversed")
+            if reversedY:
+                fig.update_yaxes(autorange="reversed")
+
+            # automatically render a png if the dataframe is too long
+            if len(inputDataFrame) > 200000 and render_type == "auto":
+                render_type = "png"
+
             # transform the fig to an png if object too big
-            if "render" in plotConfigData and "png" in plotConfigData["render"]:
+            if render_type == "png":
 
                 try:
                     # prepare to compute the image on a remote orca server
@@ -356,49 +409,62 @@ def get_plot(inputDataFrame, config, apply_parameterization=True):
                         template="plotly_white"
                     )
                     img_bytes = fig.to_image(
-                        format="png", width=1200, height=700, scale=2)
+                        format="png", width=render_size[0], height=render_size[1], scale=2)
+
                     encoded = _base64.b64encode(img_bytes)
 
                     # create an empty figure with some fixed axis
                     fig = _go.Figure()
+                    fig.add_trace(
+                        _go.Scatter(
+                            x=[0, render_size[0]],
+                            y=[0, render_size[1]],
+                            mode="markers",
+                            marker_opacity=0
+                        )
+                    )
+
+                    # Configure axes
+                    fig.update_xaxes(
+                        visible=False,
+                        range=[0, render_size[0]]
+                    )
+
+                    fig.update_yaxes(
+                        visible=False,
+                        range=[0, render_size[1]]
+                    )
+
                     fig.update_layout(
-                        xaxis=_go.layout.XAxis(
-                            showticklabels=False,
-                            showgrid=False,
-                            zeroline=False,
-                            range=[0, 1200]
-                        ),
-                        yaxis=_go.layout.YAxis(
-                            showticklabels=False,
-                            showgrid=False,
-                            zeroline=False,
-                            range=[0, 700],
-                            scaleanchor='x'
-                        ),
                         margin={'l': 0, 'r': 0, 't': 0, 'b': 0},
-                        autosize=True,
+                        # autosize=True,
                     )
 
                     # add the png to the figure
                     fig.add_layout_image(
-                        dict(
-                            source="data:image/png;base64,{}".format(
-                                encoded.decode('ascii')),
-                            xref="x",
-                            yref="y",
-                            x=0,
-                            sizex=1200,
-                            y=700,
-                            sizey=700,
-                            opacity=1.0,
-                            layer="below",
-                            sizing="stretch"
-                        )
+
+                        source="data:image/png;base64,{}".format(
+                            encoded.decode('ascii')),
+                        xref="x",
+                        yref="y",
+                        x=0,
+                        sizex=render_size[0],
+                        y=render_size[1],
+                        sizey=render_size[1],
+                        opacity=1.0,
+                        layer="below",
+                        sizing="stretch"
+
                     )
 
                 except Exception as ex:
                     print(ex)
 
+            if plotConfigData["type"] != "table":
+                fig.update_layout(
+                    template="plotly_white"
+                )
+
             return fig
 
     except Exception as inst:
